package com.tofly.feesapi.common.annotate.requestPostSingle;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.MethodParameter;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.util.*;

/**
 * @description:
 * @Title: RequestPostSingleParamMethodArgumentResolver
 * @Package com.tofly.fees.common.annotate
 * @Date 2023-06-27 9:40
 */
public class RequestPostSingleParamMethodArgumentResolver implements HandlerMethodArgumentResolver {

    private final Logger log = LoggerFactory.getLogger(RequestPostSingleParamMethodArgumentResolver.class);
    private static final String POST = "post";
    private static final String APPLICATION_JSON = "application/json";

    // private static final String APPLICATION_JSON = "x-www-form-urlencoded";
    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        // 只处理带有@RequestPostSingleParam注解的参数
        return parameter.hasParameterAnnotation(RequestPostSingleParam.class);
    }


    private String mutRequestBody = "";

    @Override
    public Object resolveArgument(MethodParameter parameter, ModelAndViewContainer mavContainer, NativeWebRequest webRequest, WebDataBinderFactory binderFactory) throws Exception {

        // 请求对象
        HttpServletRequest servletRequest = webRequest.getNativeRequest(HttpServletRequest.class);
        if (servletRequest.getParameterMap() != null &&
                servletRequest.getParameterMap().size() > 0) {
            Map<String, Object> filterMapObj = new HashMap<>();
            Map map = servletRequest.getParameterMap();
            Set key = map.keySet();
            for (Object model : key.toArray()) {
                filterMapObj.put(model.toString(), ((String[]) map.get(model))[0]);
            }
            return this.bindUrlRequestParams(parameter, filterMapObj);
        }
        // 请求对象空判
        String contentType = Objects.requireNonNull(servletRequest).getContentType();

        if (contentType == null || !contentType.contains(APPLICATION_JSON)) {
            log.error("请求: {}格式错误, contentType需为 {}", servletRequest.getRequestURL().toString(), APPLICATION_JSON);
            throw new RuntimeException("请求contentType错误");
        }

        if (!POST.equalsIgnoreCase(servletRequest.getMethod())) {
            log.error("请求: {}格式错误, 请求方式需为 {}", servletRequest.getRequestURL().toString(), POST);
            throw new RuntimeException("请求方式错误");
        }


        // 创建请求
        return this.bindRequestParams(parameter, servletRequest);
    }

    /**
     * 创建请求
     *
     * @param parameter      请求参数
     * @param servletRequest 请求对象
     * @return Object
     * @Author zhaoxz
     * @Date 2023/3/21 16:31
     */


    private Object bindRequestParams(MethodParameter parameter, HttpServletRequest servletRequest) {
        // 获取RequestPostSingleParam注解信息
        RequestPostSingleParam requestPostSingleParam = parameter.getParameterAnnotation(RequestPostSingleParam.class);
        // 参数类型
        Class<?> parameterType = parameter.getParameterType();
        // 获取body数据
        String requestBody = this.getRequestBody(servletRequest);
        // str -> json
        JSONObject paramObj = JSONObject.parseObject(requestBody);
        // 请求参数为空判定
        if (paramObj == null) {
            paramObj = new JSONObject();
        }

        // 绑定参数名称获取
        String parameterName = StringUtils.isBlank(requestPostSingleParam.value()) ? parameter.getParameterName()
                : requestPostSingleParam.value();
        // 获取参数值
        Object value = paramObj.get(parameterName);


        // 必填判定
        if (requestPostSingleParam.required()) {
            if (ObjectUtils.isEmpty(value)) {
                log.error("请求: {}, {}参数不可为空", servletRequest.getRequestURL().toString(), parameterName);
                throw new Error(String.format("参数【%s】不能为空", parameterName));
            }
        }

        try {
            switch (parameterType.getTypeName()) {
                case "java.util.HashMap":
                    value=JSONObject.parseObject(value.toString(), HashMap.class);
                    break;
                case "java.util.Map":

                    break;

            }
        } catch (Exception ex) {
            throw new Error(String.format("转换对象异常【%s】", parameterName));
        }

        return ConvertUtils.convert(value, parameterType);
    }


    private Object bindUrlRequestParams(MethodParameter parameter, Map<String, Object> map) {


        // 获取RequestPostSingleParam注解信息
        RequestPostSingleParam requestPostSingleParam = parameter.getParameterAnnotation(RequestPostSingleParam.class);

        // str -> json
        // 参数类型
        Class<?> parameterType = parameter.getParameterType();

        JSONObject paramObj = new JSONObject(map);
        // 请求参数为空判定
        if (paramObj == null) {
            paramObj = new JSONObject();
        }
        // 绑定参数名称获取
        String parameterName = StringUtils.isBlank(requestPostSingleParam.value()) ? parameter.getParameterName()
                : requestPostSingleParam.value();

        // 获取参数值
        Object value = paramObj.get(parameterName);


        switch (parameterType.getTypeName()) {
            case "java.util.ArrayList":
                String _value = (String) value;
                value = value.toString().replace("[", "").replace("]", "");
                String[] strArr = value.toString().split(",");
                ArrayList<Object> arr = new ArrayList<>();
                for (String m : strArr) {
                    arr.add(m);
                }
                value = arr;
                break;

            case "java.util.List":
                value = value.toString().replace("[", "").replace("]", "");
                String[] strList = value.toString().split(",");
                List<Object> lst = new ArrayList<>();
                for (String m : strList) {
                    lst.add(m);
                }
                value = lst;
                break;
        }


        // 必填判定
        if (requestPostSingleParam.required()) {
            if (ObjectUtils.isEmpty(value)) {
                //   log.error("请求: {}, {}参数不可为空", servletRequest.getRequestURL().toString(), parameterName);
                throw new Error(String.format("参数【%s】不能为空", parameterName));
            }
        }
        // Object setValue=ConvertUtils.convert(value, parameterType);
        Object setValue = ConvertUtils.convert(value, parameterType);
        return setValue;
    }

    /**
     * 获取请求body
     *
     * @param servletRequest 请求对象
     * @return String body信息
     * @Author zhaoxz
     * @Date 2023/3/21 16:33
     */
    private String getRequestBody(HttpServletRequest servletRequest) {
        StringBuilder stringBuilder = new StringBuilder();
        try {
            BufferedReader reader = servletRequest.getReader();
            char[] buf = new char[1024];
            int length;
            while ((length = reader.read(buf)) != -1) {
                stringBuilder.append(buf, 0, length);
            }
        } catch (IOException e) {
            log.error("请求对象读取异常", e);
            throw new RuntimeException("获取请求body数据异常");
        }
        return stringBuilder.toString();
    }

}
